let express = require("express");
let router = express.Router();
let dbCon = require("../lib/db");
const moment = require("moment");

router.get("/", (req, res, next) => {
  const workerName = req.query.worker; // Get the worker's name from query parameters
  let query = `
    SELECT tasks.*, users.name AS user_name
    FROM tasks
    LEFT JOIN users ON tasks.user_id = users.id
  `;

  // If a worker's name is provided, filter the tasks
  if (workerName) {
    query += ` WHERE users.name = ?`;
  }

  query += ` ORDER BY tasks.id ASC`;

  dbCon.query(query, [workerName], (err, rows) => {

      if (err) {
        req.flash("error", err);
        return res.render("tasks/index", { data : [], users: [] });
      }

      // ดึงข้อมูลผู้ใช้ทั้งหมด
      dbCon.query("SELECT * FROM users", (err, users) => {
        if (err) {
          req.flash("error", "ไม่สามารถดึงข้อมูลผู้ใช้ได้");
          return res.render("tasks/index", { data: rows, users: [] });
        }

        // ส่งข้อมูลไปที่หน้ารายการงาน
        res.render("tasks/index", { data: rows, users: users });
      });
    }
  );

});

// แสดงหน้าเพิ่มงาน
router.get("/add", (req, res) => {
  // ตรวจสอบว่าผู้ใช้ล็อกอินแล้วหรือไม่
  if (!req.session.user) {
    return res.redirect("/login"); // หากไม่ได้ล็อกอิน ให้รีไดเร็กต์ไปหน้า login
  }

  // ข้อมูลผู้ใช้จาก session
  const user = req.session.user;

  // กำหนดค่า task_types และ status_types
  const taskTypes = [
    "งานตาม KPI",
    "งานตาม JD",
    "งานที่ได้รับมอบหมาย",
    "งานตามยุทธศาสตร์",
    "งานตามแผนความเสี่ยง",
    "งานแผนซื้อจ้าง",
  ];
  const statusTypes = [
    "รอดำเนินการ",
    "อยู่ระหว่างดำเนินการ",
    "ดำเนินการแล้ว",
    "ยกเลิก",
  ];

  // ส่งข้อมูลผู้ใช้ที่ล็อกอินไปที่หน้าฟอร์ม Add Works
  res.render("tasks/add", {
    user: user,
    taskTypes: taskTypes,
    statusTypes: statusTypes,
    date: "",
    task_description: "",
    due_date: "",
    completion_date: "",
    task_type: "",
    assigned_by: "",
    status: "",
  });
});

router.post("/add", (req, res) => {
  let {
    date,
    task_description,
    due_date,
    completion_date,
    task_type,
    assigned_by,
    status,
  } = req.body;

  let user_id = req.session.user ? req.session.user.id : null; // Use user_id from session
  console.log("Form Data:", req.body); // Log the form data
  console.log("User ID:", user_id); // Log the user ID


  // ตรวจสอบค่าต่างๆ ในฟอร์ม
  if (!user_id || !task_description) {
    req.flash("error", "โปรดใส่ชื่อผู้ปฏิบัติงาน และ งานที่ปฏิบัติ");
    return res.redirect("/tasks/add");
  }

  // ถ้าไม่มีข้อผิดพลาด
  if (!user_id || !task_description) {
    req.flash("error", "โปรดใส่ชื่อผู้ปฏิบัติงาน และ งานที่ปฏิบัติ");
    return res.redirect("/tasks/add");
  }

  if (!user_id || !task_description) {
    req.flash("error", "โปรดใส่ชื่อผู้ปฏิบัติงาน และ งานที่ปฏิบัติ");
    return res.redirect("/tasks/add");
  }

  let form_data = {
    user_id,
    date,
    task_description,
    due_date,
    completion_date,
    task_type,
    assigned_by,
    status,
  };

  // เพิ่มข้อมูลลงในฐานข้อมูล
  dbCon.query("INSERT INTO tasks SET ?", form_data, (err) => {
    if (err) {
      req.flash("error", err);
      return res.redirect("/tasks/add");
    } else {
      req.flash("success", "เพิ่มงานเรียบร้อยแล้ว");
      return res.redirect("/tasks");
    }
  });
});

// แสดงหน้าแก้ไขงาน
router.get("/edit/:id", (req, res) => {
  let id = req.params.id;


  dbCon.query("SELECT * FROM tasks WHERE id = ?", [id], (err, taskRows) => {
    if (err || taskRows.length === 0) {
      req.flash("error", "ไม่พบงานที่มี ID = " + id);
      return res.redirect("/tasks");
    }

    // ดึงข้อมูลผู้ใช้ทั้งหมดสำหรับ dropdown
    dbCon.query("SELECT * FROM users", (err, users) => {
      if (err) {
        req.flash("error", "ไม่สามารถดึงข้อมูลผู้ใช้ได้");
        return res.redirect("/tasks");
      }

      let task = taskRows[0];
      res.render("tasks/edit", { task, users });
    });
  });
});

// อัพเดทงาน
router.post("/update/:id", (req, res) => {
  let id = req.params.id;
  let {
    user_id,
    date,
    task_description,
    due_date,
    completion_date,
    task_type,
    assigned_by,
    status,
  } = req.body;

  // ตรวจสอบข้อมูลที่จำเป็น
  if (!user_id || task_description.trim() === "") {
    req.flash("error", "โปรดเลือกผู้ปฏิบัติงานและกรอกงานที่ปฏิบัติ");
    return res.redirect("/tasks/edit/" + id);
  }

  // ข้อมูลที่ต้องการอัพเดท
  let form_data = {
    user_id,
    date,
    task_description,
    due_date,
    completion_date,
    task_type,
    assigned_by,
    status,
  };

  // อัพเดทข้อมูลในฐานข้อมูล
  dbCon.query("UPDATE tasks SET ? WHERE id = ?", [form_data, id], (err) => {
    if (err) {
      req.flash("error", "เกิดข้อผิดพลาด: " + err);
      return res.redirect("/tasks/edit/" + id);
    }
    req.flash("success", "Task successfully updated");
    return res.redirect("/tasks");
  });
});

// Delete task
router.get("/delete/:id", (req, res) => {
  let id = req.params.id;

  dbCon.query("DELETE FROM tasks WHERE id = ?", [id], (err) => {
    if (err) {
      req.flash("error", "เกิดข้อผิดพลาดในการลบงาน: " + err);
      return res.redirect("/tasks");
    } else {
      req.flash("success", "ลบงานเรียบร้อยแล้ว!");
      return res.redirect("/tasks");
    }
  });
});

module.exports = router;
